@@ -1,7 +1,7 @@
 # encoding: utf-8
 
 # Nagstamon - Nagios status monitor for your desktop
-# Copyright (C) 2008-2014 Henri Wahl <h.wahl@ifw-dresden.de> et al.
+# Copyright (C) 2008-2021 Henri Wahl <henri@nagstamon.de> et al.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -92,7 +92,7 @@ class IcingaWeb2Server(GenericServer):
         """
         GenericServer.init_HTTP(self)
 
-        if not 'Referer' in self.session.headers:
+        if self.session and not 'Referer' in self.session.headers:
             self.session.headers['Referer'] = self.monitor_cgi_url + '/icingaweb2/monitoring'
 
         # normally cookie out will be used
@@ -122,8 +122,8 @@ class IcingaWeb2Server(GenericServer):
         # define CGI URLs for hosts and services
         if self.cgiurl_hosts == self.cgiurl_services == self.cgiurl_monitoring_health == None:
             # services (unknown, warning or critical?)
-            self.cgiurl_services = {'hard': self.monitor_cgi_url + '/monitoring/list/services?service_state>0&service_state<=3&service_state_type=1&addColumns=service_last_check&format=json', \
-                                    'soft': self.monitor_cgi_url + '/monitoring/list/services?service_state>0&service_state<=3&service_state_type=0&addColumns=service_last_check&format=json'}
+            self.cgiurl_services = {'hard': self.monitor_cgi_url + '/monitoring/list/services?service_state>0&service_state<=3&service_state_type=1&addColumns=service_last_check,service_is_reachable&format=json', \
+                                    'soft': self.monitor_cgi_url + '/monitoring/list/services?service_state>0&service_state<=3&service_state_type=0&addColumns=service_last_check,service_is_reachable&format=json'}
             # hosts (up or down or unreachable)
             self.cgiurl_hosts = {'hard': self.monitor_cgi_url + '/monitoring/list/hosts?host_state>0&host_state<=2&host_state_type=1&addColumns=host_last_check&format=json', \
                                  'soft': self.monitor_cgi_url + '/monitoring/list/hosts?host_state>0&host_state<=2&host_state_type=0&addColumns=host_last_check&format=json'}
@@ -230,9 +230,12 @@ class IcingaWeb2Server(GenericServer):
                                 self.new_hosts[host_name].attempt = "HARD"
        
                         # extra duration needed for calculation
-                        duration = datetime.datetime.now() - datetime.datetime.fromtimestamp(int(h['host_last_state_change']))
-                        self.new_hosts[host_name].duration = strfdelta(duration, '{days}d {hours}h {minutes}m {seconds}s')
-                        
+                        if h['host_last_state_change'] is not None:
+                            last_change = h['host_last_state_change'] if h['host_last_state_change'] is not None else 0
+                            duration = datetime.datetime.now() - datetime.datetime.fromtimestamp(int(last_change))
+                            self.new_hosts[host_name].duration = strfdelta(duration,'{days}d {hours}h {minutes}m {seconds}s')
+                        else:
+                            self.new_hosts[host_name].duration = 'n/a'
                     del h, host_name
         except:
             import traceback
@@ -286,17 +289,8 @@ class IcingaWeb2Server(GenericServer):
                         # acknowledge needs host_description and no display name
                         self.new_hosts[host_name].real_name = s['host_name']
 
-                    #if self.use_display_name_host == False:
-                    #    # legacy Icinga adjustments
-                    #    if 'service_description' in s: service_name = s['service_description']
-                    #    elif 'description' in s: service_name = s['description']
-                    #    elif 'service' in s: service_name = s['service']
-                    #else:
-                    #    service_name = s['service_display_name']
-                    # regarding to https://github.com/HenriWahl/Nagstamon/issues/400 Icinga2 needs no legacy adjustments
                     service_name = s['service_display_name']
 
-
                     # if a service does not exist create its object
                     if not service_name in self.new_hosts[host_name].services:
                         self.new_hosts[host_name].services[service_name] = GenericService()
@@ -313,6 +307,10 @@ class IcingaWeb2Server(GenericServer):
                         self.new_hosts[host_name].services[service_name].acknowledged = bool(int(s['service_acknowledged']))
                         self.new_hosts[host_name].services[service_name].scheduled_downtime = bool(int(s['service_in_downtime']))
                         self.new_hosts[host_name].services[service_name].status_type = status_type
+                        self.new_hosts[host_name].services[service_name].unreachable = s['service_is_reachable'] == '0'
+
+                        if self.new_hosts[host_name].services[service_name].unreachable:
+                            self.new_hosts[host_name].services[service_name].status_information += " (SERVICE UNREACHABLE)"
                         
                         # extra Icinga properties to solve https://github.com/HenriWahl/Nagstamon/issues/192
                         # acknowledge needs service_description and no display name
@@ -328,9 +326,13 @@ class IcingaWeb2Server(GenericServer):
                                 self.new_hosts[host_name].services[service_name].attempt = "HARD"
                         
                         # extra duration needed for calculation
-                        duration = datetime.datetime.now() - datetime.datetime.fromtimestamp(int(s['service_last_state_change']))
-                        self.new_hosts[host_name].services[service_name].duration = strfdelta(duration, '{days}d {hours}h {minutes}m {seconds}s')                      
-                        
+                        if s['service_last_state_change'] is not None:
+                            last_change = s['service_last_state_change'] if s['service_last_state_change'] is not None else 0
+                            duration = datetime.datetime.now() - datetime.datetime.fromtimestamp(int(last_change))
+                            self.new_hosts[host_name].services[service_name].duration = strfdelta(duration, '{days}d {hours}h {minutes}m {seconds}s')
+                        else:
+                            self.new_hosts[host_name].services[service_name].duration = 'n/a'
+
                     del s, host_name, service_name
         except:
 
