@@ -1,6 +1,6 @@
 # -*- encoding: utf-8; py-indent-offset: 4 -*-
 #
-# Zabbix.py based on Check_MK Multisite.py
+# Zabbix.py based on Checkmk Multisite.py
 
 import sys
 import urllib.request
@@ -84,15 +84,27 @@ class ZabbixServer(GenericServer):
 
     def _login(self):
         try:
-            self.zapi = ZabbixAPI(server=self.monitor_url, path="", log_level=self.log_level,
-                                  validate_certs=self.validate_certs)
-            self.zapi.login(self.username, self.password)
+            # create ZabbixAPI if not yet created
+            if self.zapi is None:
+                self.zapi = ZabbixAPI(server=self.monitor_url, path="", log_level=self.log_level, validate_certs=self.validate_certs)
+            # login if not yet logged in, or if login was refused previously
+            if not self.zapi.logged_in():
+                self.zapi.login(self.username, self.password)
         except ZabbixAPIException:
             result, error = self.Error(sys.exc_info())
             return Result(result=result, error=error)
 
     def getLastApp(self, this_item):
         if len(this_item) > 0:
+            if "applications" not in this_item[0]:
+                if 'tags' in this_item[0]:
+                    app = "NO APP"
+                    for tag in this_item[0]['tags']:
+                        if tag['tag'] == 'Application':
+                            app = tag['value']
+                    return app
+                else:
+                    return "NO APP"
             last_app = len(this_item[0]['applications']) - 1  # use it to get the last application name
             if last_app > -1:
                 return "%s" % this_item[0]['applications'][last_app]['name']
@@ -112,8 +124,7 @@ class ZabbixServer(GenericServer):
         nagitems = {"services": [], "hosts": []}
 
         # Create URLs for the configured filters
-        if self.zapi is None:
-            self._login()
+        self._login()
 
         try:
             # =========================================
@@ -144,7 +155,7 @@ class ZabbixServer(GenericServer):
             # ipmi_available IPMI: 0 -> No agents 1 -> available 2-> Agent access error
             # maintenance_status = 1 In maintenance
             for host in hosts:
-                
+
                 n = {
                     'host': host['host'],
                     'name': host['name'],
@@ -163,9 +174,11 @@ class ZabbixServer(GenericServer):
                     # Zabbix backend data
                     'hostid': host['hostid'],
                     'site': '',
-                    'address': host['interfaces'][0]['ip'],
+                    # 'address': host['interfaces'][0]['ip'],
                 }
 
+                # try to fix https://github.com/HenriWahl/Nagstamon/issues/687
+                n['address'] = host['interfaces'][0]['ip'] if len(host['interfaces']) > 0 else ''
 
                 if host['maintenance_status'] == '1':
                     n['scheduled_downtime'] = True
@@ -175,28 +188,28 @@ class ZabbixServer(GenericServer):
                     n['notifications_disabled'] = True
                     # Filter only hosts by filter "Host & services with disabled checks"
                     n['passiveonly']           = True
-
-                if host['available'] == '0' and host['snmp_available'] == '0' and host['ipmi_available'] == '0' and host['jmx_available'] == '0':
-                    n['status']             = 'UNREACHABLE'
-                    n['status_information'] = 'Host agents in unknown state'
-                    n['duration']           = 'Unknown' 
-                if host['ipmi_available'] == '2':
+                # attempt to fix https://github.com/HenriWahl/Nagstamon/issues/535
+                # if host['available'] == '0' and host['snmp_available'] == '0' and host['ipmi_available'] == '0' and host['jmx_available'] == '0':
+                #     n['status']             = 'UNREACHABLE'
+                #     n['status_information'] = 'Host agents in unknown state'
+                #     n['duration']           = 'Unknown'
+                if host.get('ipmi_available', '0') == '2':
                     n['status']             = 'DOWN'
                     n['status_information'] = host['ipmi_error']
                     n['duration']           = HumanReadableDurationFromTimestamp(host['ipmi_errors_from'])
-                if host['snmp_available'] == '2':
+                if host.get('snmp_available', '0') == '2':
                     n['status']             = 'DOWN'
                     n['status_information'] = host['snmp_error']
                     n['duration']           = HumanReadableDurationFromTimestamp(host['snmp_errors_from'])
-                if host['jmx_available'] == '2':
+                if host.get('jmx_available', '0') == '2':
                     n['status']             = 'DOWN'
                     n['status_information'] = host['jmx_error']
                     n['duration']           = HumanReadableDurationFromTimestamp(host['jmx_errors_from'])
-                if host['available'] == '2':
+                if host.get('available', '0') == '2':
                     n['status']             = 'DOWN'
                     n['status_information'] = host['error']
                     n['duration']           = HumanReadableDurationFromTimestamp(host['errors_from'])
-                
+
                 # Zabbix shows OK hosts too - kick 'em!
                 if not n['status'] == 'UP':
                     # add dictionary full of information about this host item to nagitems
@@ -274,16 +287,27 @@ class ZabbixServer(GenericServer):
                                                         'withLastEventUnacknowledged': True,
                                                   })
                 unack_trigger_ids = [u['triggerid'] for u in unack_triggers]
-                
+
+                # prefetch items
+                all_item_ids = list(set([t['items'][0]['itemid'] for t in triggers]))
+                all_items = self.zapi.item.get(
+                    {'itemids':all_item_ids,
+                        'output': ['itemid', 'hostid', 'name', 'lastvalue'],
+                        'selectTags': 'extend',
+                        'selectApplications': 'extend'}
+                )
+                itemid_item_map = {i['itemid']: i for i in all_items}
+
                 for t in triggers:
                     t['acknowledged'] = False if t['triggerid'] in unack_trigger_ids else True
-                    
+
                     # get Application name for the trigger
-                    this_item = self.zapi.item.get(
-                        {'itemids': [t['items'][0]['itemid']],
-                         'output': ['itemid', 'hostid', 'name', 'lastvalue'],
-                         'selectApplications': 'extend'}
-                    )
+                    if t['items'][0]['itemid'] in itemid_item_map:
+                        this_item = [itemid_item_map[t['items'][0]['itemid']]]
+                    else:
+                        # This else condition should never be hit, except in rare circumstances the trigger/item
+                        # config is updated at the same time Nagstamon is pulling trigger/item config
+                        this_item = []
                     t['application'] = self.getLastApp(this_item)
                     try:
                         t['lastvalue']   = this_item[0]['lastvalue']
@@ -368,7 +392,7 @@ class ZabbixServer(GenericServer):
                         n['scheduled_downtime'] = True
 
                     nagitems["services"].append(n)
-                    
+
                     # after collection data in nagitems create objects of its informations
                     # host objects contain service objects
                     # if not created previously, host should be created with proper data
@@ -383,7 +407,7 @@ class ZabbixServer(GenericServer):
                     new_service = n["triggerid"]
                     if new_service not in self.new_hosts[key].services:
                         self.new_hosts[key].services[new_service] = GenericService()
-                        
+
                         self.new_hosts[key].services[new_service].host       = n["hostname"] if len(n['hostname']) != 0 else n["host"]
                         self.new_hosts[key].services[new_service].name       = n["service"]
                         self.new_hosts[key].services[new_service].status     = n["status"]
@@ -410,7 +434,7 @@ class ZabbixServer(GenericServer):
             result, error = self.Error(sys.exc_info())
             print(sys.exc_info())
             return Result(result=result, error=error)
-        
+
         return ret
 
     def _open_browser(self, url):
@@ -453,13 +477,12 @@ class ZabbixServer(GenericServer):
         # Service column is storing current trigger id
         triggerids = []
         triggerids.append(service)
-        
+
         # acknowledge all problems (column services) on a host when told to do so
         for s in all_services:
             triggerids.append(s)
 
-        if self.zapi is None:
-            self._login()
+        self._login()
 
         for triggerid in triggerids:
             events = []
@@ -470,37 +493,81 @@ class ZabbixServer(GenericServer):
                                           'sortfield': 'clock',
                                           'sortorder': 'DESC'}):
                 # Get only current event status, but retrieving first row ordered by clock DESC
-                
+
                 # If event status is not "OK" (Still is an active problem), mark event to acknowledge/close
                 if e['value'] != '0':
                     events.append(e['eventid'])
                 # Only take care of newest event, discard all next
                 break
-                
+
             # If events pending of acknowledge, execute ack
             if len(events) > 0:
+                # actions is a bitmask with values:
+                # 1 - close problem
+                # 2 - acknowledge event
+                # 4 - add message
+                # 8 - change severity
+                # 16 - unacknowledge event
+                actions = 0
                 # If sticky is set then close only current event
-                close = 1 if (triggerid == service and sticky is True) else 0
+                if triggerid == service and sticky:
+                    # do not send the "Close" flag if this event does not allow manual closing
+                    triggers = self.zapi.trigger.get({
+                        'output': ['triggerid', 'manual_close'],
+                        'filter': {'triggerid': triggerid}})
+                    if not triggers or 'manual_close' not in triggers[0] or str(triggers[0]['manual_close']) == '1':
+                        actions |= 1
+                # The current Nagstamon menu items don't match up too well with the Zabbix actions,
+                # but perhaps "Persistent comment" is the closest thing to acknowledgement
+                if persistent:
+                    actions |= 2
+                if comment:
+                    actions |= 4
                 if conf.debug_mode is True:
-                    self.Debug(server=self.get_name(), debug="Events to acknowledge: " + str(events) + " Close: " + str(close))
-                self.zapi.event.acknowledge({'eventids': events, 'message': '%s: %s' % (author, comment), 'action': close})
- 
+                    self.Debug(server=self.get_name(), debug="Events to acknowledge: " + str(events) + " Close: " + str(actions))
+                self.zapi.event.acknowledge({'eventids': events, 'message': comment, 'action': actions})
+
     def _set_downtime(self, hostname, service, author, comment, fixed, start_time, end_time, hours, minutes):
 
+        # Check if there is an associated Application tag with this trigger/item
+        app = None
+        triggers = self.zapi.trigger.get({
+            'selectItems': ['itemid'],
+            'output': ['triggerid'],
+            'filter': {'triggerid': service}})
+
+        if triggers and triggers[0]['items']:
+            items = self.zapi.item.get({
+                'itemids': [triggers[0]['items'][0]['itemid']],
+                'output': ['itemid'],
+                'selectTags': 'extend'})
+            if items and items[0]['tags']:
+                for tag in items[0]['tags']:
+                    if tag['tag'] == 'Application':
+                        app = tag['value']
+
         hostids = [ self.hosts[hostname].hostid ]
-        
+
         date  = datetime.datetime.strptime(start_time, "%Y-%m-%d %H:%M")
         stime = time.mktime(date.timetuple())
 
         date  = datetime.datetime.strptime(end_time, "%Y-%m-%d %H:%M")
         etime = time.mktime(date.timetuple())
-        
+
         if conf.debug_mode is True:
             self.Debug(server=self.get_name(), debug="Downtime for " + hostname + "[" + str(hostids[0]) + "] stime:" + str(stime) + " etime:" + str(etime))
-            
-        self.zapi.maintenance.create({'hostids': hostids, 'name': comment, 'description': author, 'active_since': stime, 'active_till': etime, 'maintenance_type' : 0, "timeperiods": [
+
+        body = {'hostids': hostids, 'name': comment, 'description': author, 'active_since': stime, 'active_till': etime, 'maintenance_type' : 0, "timeperiods": [
             { "timeperiod_type": 0, "start_date": stime, "period": etime - stime }
-        ]})
+        ]}
+        if app:
+            body['tags'] = [{'tag': 'Application', 'operator': 0, 'value': app}]
+            body['description'] += ' ' + body['name']
+            body['name'] = f'{start_time} {hostname} {app}'
+        try:
+            self.zapi.maintenance.create(body)
+        except Already_Exists:
+            self.Debug(server=self.get_name(), debug=f"Maintanence with name {body['name']} already exists")
 
     def get_start_end(self, host):
         return time.strftime("%Y-%m-%d %H:%M"), time.strftime("%Y-%m-%d %H:%M", time.localtime(time.time() + 7200))
@@ -544,4 +611,4 @@ class ZabbixServer(GenericServer):
         if (" on " or " is ") in service:
             for separator in [" on ", " is "]:
                 service = service.split(separator)[0]
-        return(service)
\ No newline at end of file
+        return(service)
