@@ -2,7 +2,7 @@
 # encoding: utf-8
 
 # Nagstamon - Nagios status monitor for your desktop
-# Copyright (C) 2008-2020 Henri Wahl <h.wahl@ifw-dresden.de> et al.
+# Copyright (C) 2008-2021 Henri Wahl <henri@nagstamon.de> et al.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ import zlib
 import datetime
 from urllib.parse import quote
 from collections import OrderedDict
+from pathlib import Path
 
 # older Kubuntu has trouble with keyring
 # see https://github.com/HenriWahl/Nagstamon/issues/447
@@ -114,6 +115,7 @@ CONFIG_STRINGS = ['custom_browser',
                   'proxy_password',
                   'autologin_key',
                   'custom_cert_ca_file',
+                  'idp_ecp_endpoint',
                   'monitor_site'
                   ]
 
@@ -123,18 +125,23 @@ class AppInfo(object):
         contains app information previously located in GUI.py
     """
     NAME = 'Nagstamon'
-    VERSION = '3.4.1'
-    WEBSITE = 'https://nagstamon.ifw-dresden.de'
-    COPYRIGHT = '©2008-2020 Henri Wahl et al.'
+    VERSION = '3.8.0'
+    WEBSITE = 'https://nagstamon.de'
+    COPYRIGHT = '©2008-2021 Henri Wahl et al.'
     COMMENTS = 'Nagios status monitor for your desktop'
+    # dict of servers to offer for downloads if an update is available
+    DOWNLOAD_SERVERS = {'nagstamon.de': 'https://github.com/HenriWahl/Nagstamon/releases',
+                        'nagstamon.ifw-dresden.de': 'https://nagstamon.ifw-dresden.de/download'}
     # version URL depends on version string
     if 'alpha' in VERSION.lower() or \
         'beta' in VERSION.lower() or \
         'rc' in VERSION.lower() or \
         '-' in VERSION.lower():
         VERSION_URL = WEBSITE + '/version/unstable'
+        VERSION_PATH = '/version/unstable'
     else:
         VERSION_URL = WEBSITE + '/version/stable'
+        VERSION_PATH = '/version/stable'
 
 
 class Config(object):
@@ -160,6 +167,7 @@ class Config(object):
         self.default_sort_order = 'descending'
         self.filter_all_down_hosts = False
         self.filter_all_unreachable_hosts = False
+        self.filter_all_unreachable_services = False
         self.filter_all_flapping_hosts = False
         self.filter_all_unknown_services = False
         self.filter_all_information_services = False
@@ -297,8 +305,11 @@ class Config(object):
         # internal flag to determine if keyring is available at all - defaults to False
         # use_system_keyring is checked and defined some lines later after config file was read
         self.keyring_available = False
-        # setting for keyring usage
-        self.use_system_keyring = False
+        # setting for keyring usage - might cause trouble on Linux so disable it there as default to avoid crash at start
+        if OS in OS_NON_LINUX:
+            self.use_system_keyring = True
+        else:
+            self.use_system_keyring = False
 
         # Special FX
         # Centreon
@@ -469,7 +480,7 @@ class Config(object):
                 elif servers[server].proxy_password != "":
                     servers[server].proxy_password = self.DeObfuscate(servers[server].proxy_password)
 
-                # do only deobfuscating if any autologin_key is set - will be only Centreon
+                # do only deobfuscating if any autologin_key is set - will be only Centreon/Thruk
                 if 'autologin_key' in servers[server].__dict__.keys():
                     if len(servers[server].__dict__['autologin_key']) > 0:
                         servers[server].autologin_key = self.DeObfuscate(servers[server].autologin_key)
@@ -702,7 +713,7 @@ class Config(object):
                 # import secretstorage module as dependency of keyring -
                 # if not available keyring won't work
                 import secretstorage
-                if ("SecretService") in dir(keyring.backends) and not (keyring.get_keyring() is None):
+                if keyring.get_keyring():
                     return True
                 else:
                     return False
@@ -827,7 +838,7 @@ class Config(object):
         defaultactions["Checkmk Edit host in WATO"] = Action(name="Checkmk Edit host in WATO", enabled=False,
                                                               monitor_type="Checkmk Multisite",
                                                               description="Edit host in WATO.",
-                                                              string="$MONITOR$index.py?start_url=%2Fmonitor%2Fcheck_mk%2Fwato.py%3Fhost%3D$HOST$%26mode%3Dedit_host")
+                                                              string="$MONITOR$/wato.py?host=$HOST$&mode=edit_host")
         defaultactions["Email"] = Action(name="Email", enabled=False, description="Send email to someone.",
                                          type="browser",
                                          string="mailto:servicedesk@my.org?subject=Monitor alert: $HOST$ - $SERVICE$ - $STATUS-INFO$&body=Please help!.%0d%0aBest regards from Nagstamon")
@@ -923,6 +934,8 @@ class Server(object):
         self.custom_cert_use = False
         self.custom_cert_ca_file = ''
 
+        self.idp_ecp_endpoint = 'https://idp/idp/profile/SAML2/SOAP/ECP'
+
         # special FX
         # Centreon autologin
         self.use_autologin = False
@@ -952,6 +965,16 @@ class Server(object):
         # Zabbix "Item Description" as "Service Name"
         self.use_description_name_service = False
 
+        # Prometheus/Alertmanager mappings
+        self.map_to_hostname = "pod_name,namespace,instance"
+        self.map_to_servicename = "alertname"
+        self.map_to_status_information = "message,summary,description"
+        # Alertmanager mappings
+        self.alertmanager_filter = ''
+        self.map_to_critical = 'critical,error'
+        self.map_to_warning = 'warning,warn'
+        self.map_to_unknown = 'unknown'
+        self.map_to_ok = 'ok'
 
 class Action(object):
     """
@@ -1060,3 +1083,5 @@ except Exception as err:
         if os.path.exists(path):
             RESOURCES = path
             break
+    else:
+        RESOURCES = str(Path(__file__).parent.absolute().joinpath('resources'))
