@@ -1,7 +1,7 @@
 # encoding: utf-8
 
 # Nagstamon - Nagios status monitor for your desktop
-# Copyright (C) 2008-2020 Henri Wahl <h.wahl@ifw-dresden.de> et al.
+# Copyright (C) 2008-2021 Henri Wahl <henri@nagstamon.de> et al.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -62,6 +62,11 @@ from Nagstamon.Config import (AppInfo,
 #except ImportError:
 #    from requests_kerberos import HTTPKerberosAuth as HTTPSKerberos
 
+try:
+    from requests_ecp import HTTPECPAuth
+except:
+    pass
+
 # disable annoying SubjectAltNameWarning warnings
 try:
     from requests.packages.urllib3.exceptions import SubjectAltNameWarning
@@ -182,6 +187,9 @@ class GenericServer(object):
         # flag which tells GUI if there is an TLS problem
         self.tls_error = False
 
+        # counter for login attempts - have to be threaten differently by every monitoring server type
+        self.login_count = 0
+
         # to handle Icinga versions this information is necessary, might be of future use for others too
         self.version = ''
 
@@ -245,21 +253,25 @@ class GenericServer(object):
             'soft': self.monitor_cgi_url + '/status.cgi?hostgroup=all&style=hostdetail&hoststatustypes=12&hostprops=524288&limit=0'}
 
     def init_HTTP(self):
-        '''
-            partly not constantly working Basic Authorization requires extra Authorization headers,
-            different between various server types
-        '''
-        if self.session is None:
+        """
+        partly not constantly working Basic Authorization requires extra Authorization headers,
+        different between various server types
+        """
+        if self.refresh_authentication:
+            self.session = None
+            return False
+        elif self.session is None:
             self.session = requests.Session()
             self.session.headers['User-Agent'] = self.USER_AGENT
 
             # support for different authentication types
             if self.authentication == 'basic':
                 # basic authentication
-                # self.session.auth = (self.username, self.password)
                 self.session.auth = requests.auth.HTTPBasicAuth(self.username, self.password)
             elif self.authentication == 'digest':
                 self.session.auth = requests.auth.HTTPDigestAuth(self.username, self.password)
+            elif self.authentication == 'ecp':
+                self.session.auth = HTTPECPAuth(self.idp_ecp_endpoint, username=self.username, password=self.password)
 #            elif self.authentication == 'kerberos':
 #                self.session.auth = HTTPSKerberos()
 
@@ -273,6 +285,7 @@ class GenericServer(object):
 
             # add proxy information
             self.proxify(self.session)
+            return True
 
     def proxify(self, requester):
         '''
@@ -297,7 +310,7 @@ class GenericServer(object):
                 host_port = ''.join(proxy_address_parts[1:])
 
                 # use only valid schemes
-                if scheme.lower() in ('http:', 'https:'):
+                if scheme.lower() in ('http:', 'https:', 'socks5:', 'socks5h:'):
                     # merge proxy URL
                     proxy_url = '{0}//{1}{2}'.format(scheme, user_pass, host_port)
                     # fill session.proxies for both protocols
@@ -1075,6 +1088,12 @@ class GenericServer(object):
                                        service.name))
                     service.visible = False
 
+                if conf.filter_all_unreachable_services is True and service.unreachable is True:
+                    if conf.debug_mode:
+                        self.Debug(server=self.get_name(), debug='Filter: UNREACHABLE ' + str(host.name) + ';' + str(
+                                       service.name))
+                    service.visible = False
+
                 # Checkmk and OP5 do not show the status_type so their host.status_type will be empty
                 if service.status_type != '':
                     if conf.filter_services_in_soft_state is True and service.status_type == 'soft':
@@ -1082,7 +1101,8 @@ class GenericServer(object):
                             self.Debug(server=self.get_name(),
                                        debug='Filter: SOFT STATE ' + str(host.name) + ';' + str(service.name))
                         service.visible = False
-                else:
+                # fix for https://github.com/HenriWahl/Nagstamon/issues/654
+                elif not self.TYPE.startswith("Zabbix"):
                     if len(service.attempt) < 3:
                         service.visible = True
                     elif len(service.attempt) == 3:
@@ -1425,7 +1445,7 @@ class GenericServer(object):
                         cgi_data_log = cgi_data
                     self.Debug(server=self.get_name(), debug='FetchURL: ' + url + ' CGI Data: ' + str(cgi_data_log))
 
-                if OS == OS_DARWIN and not self.cacert_path.is_file():
+                if OS == OS_DARWIN and self.cacert_path and not self.cacert_path.is_file():
                     # pyinstaller temp folder seems to be emptied completely after a while
                     # so the directories containing the resources have to be recreated too
                     self.cacert_path.parent.mkdir(exist_ok=True)
@@ -1445,7 +1465,7 @@ class GenericServer(object):
 
                 # use session only for connections to monitor servers, other requests like looking for updates
                 # should go out without credentials
-                if no_auth is False:
+                if no_auth is False and not self.refresh_authentication:
                     # check if there is really a session
                     if not self.session:
                         self.reset_HTTP()
@@ -1473,9 +1493,6 @@ class GenericServer(object):
                     # add proxy information if necessary
                     self.proxify(temporary_session)
 
-                    # no need to check TLS validity for temporary sessions like update check
-                    ###temporary_session.verify = False
-
                     # most requests come without multipart/form-data
                     if multipart is False:
                         if cgi_data is None:
